package no.ntnu.idatt1002.demo.data.Economics;

import org.junit.jupiter.api.*;

import java.io.FileNotFoundException;
import java.io.IOException;
import static org.junit.jupiter.api.Assertions.*;

class FileHandlingTest {
    FileHandling fileHandling = new FileHandling();
    ExpenseRegister expenseRegister = new ExpenseRegister();
    IncomeRegister incomeRegister = new IncomeRegister();
    @Nested
    @DisplayName("FileHandling IncomeRegister to file")
    class fileHandlingIncomeRegisterToFile{
        String fileTitle = "incomeRegisterTest";
        @Nested
        @DisplayName("FileHandling incomeRegister with income that has description")
        class fileHandlingIncomeRegisterWithIncomeThatHasDescription{
            @BeforeEach
            void addIncomeToIncomeRegister(){
                Income income1 = new Income("description", 59.9f, false, IncomeCategory.GIFT, "03.03.23");
                incomeRegister.addItem(income1);
            }
            @Test
            @DisplayName("Writing to file does not throw exception")
            void writingToFileDoesNotThrowException() throws IOException {
                assertDoesNotThrow(()->fileHandling.writeItemRegisterToFile(incomeRegister, fileTitle));
            }

            @Test
            @DisplayName("Reading from file gives correct incomeRegister back")
            void readingFromFileGivesCorrectIncomeRegisterBack() throws IOException {
                assertEquals(incomeRegister.toString(), fileHandling.readIncomeRegisterFromFile(fileTitle).toString());
            }
        }
        @Nested
        @DisplayName("FileHandling incomeRegister with income that does not have description")
        class fileHandlingIncomeRegisterWithIncomeThatDoesNotHaveDescription{
            @BeforeEach
            void addIncomeToIncomeRegister(){
                Income income1 = new Income(59.9f, false, IncomeCategory.GIFT, "03.03.23");
                incomeRegister.addItem(income1);
            }
            @Test
            @DisplayName("Writing to file does not throw exception")
            void writingToFileDoesNotThrowException() throws IOException {
                assertDoesNotThrow(()->fileHandling.writeItemRegisterToFile(incomeRegister, fileTitle));
            }

            @Test
            @DisplayName("Reading from file gives correct incomeRegister back")
            void readingFromFileGivesCorrectIncomeRegisterBack() throws IOException {
                assertEquals(incomeRegister.toString(), fileHandling.readIncomeRegisterFromFile(fileTitle).toString());
            }
        }
        @Nested
        @DisplayName("FileHandling incomeRegister with multiple Income")
        class fileHandlingIncomeRegisterWithMultipleIncome{
            @BeforeEach
            void addIncomeToIncomeRegister(){
                Income income1 = new Income(59.9f, false, IncomeCategory.GIFT, "03.03.23");
                Income income2 = new Income("description2", 62.4f, true, IncomeCategory.GIFT, "01.02.21");
                Income income3 = new Income("description3", 9.81f, false, IncomeCategory.SALARY, "05.07.23");
                incomeRegister.addItem(income1);
                incomeRegister.addItem(income2);
                incomeRegister.addItem(income3);
            }
            @Test
            @DisplayName("Writing to file does not throw exception")
            void writingToFileDoesNotThrowException() throws IOException {
                assertDoesNotThrow(()->fileHandling.writeItemRegisterToFile(incomeRegister, fileTitle));
            }

            @Test
            @DisplayName("Reading from file gives correct incomeRegister back")
            void readingFromFileGivesCorrectIncomeRegisterBack() throws IOException {
                assertEquals(incomeRegister.toString(), fileHandling.readIncomeRegisterFromFile(fileTitle).toString());
            }
        }
    }
    @Nested
    @DisplayName("FileHandling ExpenseRegister to file")
    class fileHandlingExpenseRegisterToFile{
        String fileTitle = "expenseRegisterTest";
        @Nested
        @DisplayName("FileHandling ExpenseRegister with Expense that has description")
        class fileHandlingExpenseRegisterWithExpenseThatHasDescription{
            @BeforeEach
            void addExpenseToExpenseRegister(){
                Expense expense1 = new Expense("description", 59.9f, false, ExpenseCategory.CLOTHES, "03.03.23");
                expenseRegister.addItem(expense1);
            }
            @Test
            @DisplayName("Writing to file does not throw exception")
            void writingToFileDoesNotThrowException() throws IOException {
                assertDoesNotThrow(()->fileHandling.writeItemRegisterToFile(expenseRegister, fileTitle));
            }

            @Test
            @DisplayName("Reading from file gives correct expenseRegister back")
            void readingFromFileGivesCorrectIncomeRegisterBack() throws IOException {
                assertEquals(expenseRegister.toString(), fileHandling.readExpenseRegisterFromFile(fileTitle).toString());
            }
        }
        @Nested
        @DisplayName("FileHandling ExpenseRegister with Expense that does not have description")
        class fileHandlingExpenseRegisterWithExpenseThatDoesNotHaveDescription{
            @BeforeEach
            void addExpenseToExpenseRegister(){
                Expense expense1 = new Expense(59.9f, false, ExpenseCategory.CLOTHES, "03.03.23");
                expenseRegister.addItem(expense1);
            }
            @Test
            @DisplayName("Writing to file does not throw exception")
            void writingToFileDoesNotThrowException() throws IOException {
                assertDoesNotThrow(()->fileHandling.writeItemRegisterToFile(expenseRegister, fileTitle));
            }

            @Test
            @DisplayName("Reading from file gives correct expenseRegister back")
            void readingFromFileGivesCorrectIncomeRegisterBack() throws IOException {
                assertEquals(expenseRegister.toString(), fileHandling.readExpenseRegisterFromFile(fileTitle).toString());
            }
        }
        @Nested
        @DisplayName("FileHandling ExpenseRegister with multiple Expense")
        class fileHandlingExpenseRegisterWithMultipleExpensen{
            @BeforeEach
            void addExpenseToExpenseRegister(){
                Expense expense1 = new Expense(59.9f, false, ExpenseCategory.CLOTHES, "03.03.23");
                Expense expense2 = new Expense("description2", 62.4f, true, ExpenseCategory.FOOD, "01.02.21");
                Expense expense3 = new Expense("description3", 9.81f, false, ExpenseCategory.CLOTHES, "05.07.23");

                expenseRegister.addItem(expense1);
                expenseRegister.addItem(expense2);
                expenseRegister.addItem(expense3);
            }
            @Test
            @DisplayName("Writing to file does not throw exception")
            void writingToFileDoesNotThrowException() throws IOException {
                assertDoesNotThrow(()->fileHandling.writeItemRegisterToFile(expenseRegister, fileTitle));
            }

            @Test
            @DisplayName("Reading from file gives correct expenseRegister back")
            void readingFromFileGivesCorrectIncomeRegisterBack() throws IOException {
                assertEquals(expenseRegister.toString(), fileHandling.readExpenseRegisterFromFile(fileTitle).toString());
            }
        }
    }
}